import torch
import torchvision
from matplotlib import pyplot as plt

class FashionMNIST: 
    """ Adapted from Dive into Deep Learning """
    def __init__(self, batch_size=64, resize=(28, 28)):
        transform = torchvision.transforms.Compose([
                            torchvision.transforms.Resize(resize),
                            torchvision.transforms.ToTensor()])
        self.batch_size = batch_size
        # FashionMNIST has two splits, train and test. We use the test as
        # validation.
        self.train = torchvision.datasets.FashionMNIST(root='./',
                        train=True, transform=transform, download=True)
        self.val = torchvision.datasets.FashionMNIST(root='./',
                        train=False, transform=transform, download=True)

    def text_labels(self, indices):
        labels = ['t-shirt', 'trousers', 'pullover', 'dress', 'coat', 'sandal',
                  'shirt', 'sneaker', 'bag', 'ankle boot']
        return [labels[int(i)] for i in indices]

    def get_dataloader(self, train=True):
        data = self.train if train else self.val
        return torch.utils.data.DataLoader(data, self.batch_size, shuffle=train)

    def plot(self, images, labels):
        """ Works alright with up to ten images """ 
        ncols = 5
        fig = plt.figure()
        n = images.size(0)
        for i in range(n):
            ax = fig.add_subplot((i//ncols)+1, ncols, (i%ncols)+1)
            plt.imshow(images[i].squeeze(0))
            ax.title.set_text(labels[i])
        plt.show()


def valid_loss(model, data):
    return -1

def train(model, data, epochs=10): 

    optimizer = torch.optim.SGD(model.parameters(), lr=0.001, momentum=0.9)
    cost_func = torch.nn.NLLLoss()
    for epoch in range(1, epochs+1):
        epoch_loss = 0.
        running_loss = 0.
        for i, d in enumerate(data.get_dataloader()):
            inputs, labels = d
            y_pred = model(inputs)
            loss = cost_func(torch.log(y_pred), labels)

            epoch_loss += loss.item()
            running_loss += loss.item()

            if i % 100 == 99:
                avg_batch_loss = running_loss/100
                running_loss = 0
                print(f"batch {i+1} loss: {avg_batch_loss}")

            optimizer.zero_grad()
            loss.backward()
            optimizer.step()
        val_loss = valid_loss(model, data.get_dataloader(train=False))
        print(f"Epoch {epoch} Train Avg Loss {epoch_loss/i} Validation",
              f" Loss {val_loss}")
        print('-'*80)

if __name__ == '__main__':
    data = FashionMNIST(resize=(32, 32))
